from .csfdsk_movie_review_sentiment_classification import (
    CSFDSKMovieReviewSentimentClassification,
    CSFDSKMovieReviewSentimentClassificationV2,
)
from .slovak_hate_speech_classification import (
    SlovakHateSpeechClassification,
    SlovakHateSpeechClassificationV2,
)
from .slovak_movie_review_sentiment_classification import (
    SlovakMovieReviewSentimentClassification,
    SlovakMovieReviewSentimentClassificationV2,
)

__all__ = [
    "CSFDSKMovieReviewSentimentClassification",
    "CSFDSKMovieReviewSentimentClassificationV2",
    "SlovakHateSpeechClassification",
    "SlovakHateSpeechClassificationV2",
    "SlovakMovieReviewSentimentClassification",
    "SlovakMovieReviewSentimentClassificationV2",
]



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DutchNewsArticlesClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DutchNewsArticlesClassification",
        dataset={
            "path": "clips/mteb-nl-news-articles-cls",
            "revision": "0a7227d31f85c5676be92767f8df5405ea93de54",
        },
        description="This dataset contains all the articles published by the NOS as of the 1st of January 2010. The "
        "data is obtained by scraping the NOS website. The NOS is one of the biggest (online) news "
        "organizations in the Netherlands.",
        reference="https://www.kaggle.com/datasets/maxscheijen/dutch-news-articles",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="f1",
        date=("2009-11-01", "2010-01-01"),
        domains=["Written", "News"],
        task_subtypes=["Topic classification"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        prompt="Classificeer het gegeven nieuwsartikel in het juiste onderwerp of thema",
    )



from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta
from mteb.types import PromptType


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    if not instruction or prompt_type == PromptType.document:
        return ""
    if isinstance(instruction, dict):
        if prompt_type is None:
            instruction = next(iter(instruction.values()))  # TODO
        else:
            instruction = instruction[prompt_type]
    return f"Instruct: {instruction}\nQuery:"


training_data = {
    "T2Retrieval",
    "DuRetrieval",
    "MMarcoReranking",
    "CMedQAv2-reranking",
    "NQ",
    "MSMARCO",
    "HotpotQA",
    "MrTidyRetrieval",
    "MIRACLRetrieval",
    "CodeSearchNet",
}


yuan_embedding_2_en = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        apply_instruction_to_passages=False,
    ),
    name="IEITYuan/Yuan-embedding-2.0-en",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="b2fd15da3bcae3473c8529593825c15068f09fce",
    release_date="2025-11-27",
    n_parameters=595776512,
    n_embedding_parameters=155309056,
    memory_usage_mb=2272,
    embed_dim=1024,
    max_tokens=2048,
    license="apache-2.0",
    reference="https://huggingface.co/IEITYuan/Yuan-embedding-2.0-en",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=training_data,
    adapted_from="Qwen/Qwen3-Embedding-0.6B",
)



"""tests for the MTEB CLI"""

import subprocess
import sys
from argparse import Namespace
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest
import yaml

from mteb.cli.build_cli import (
    _available_benchmarks,
    _available_tasks,
    _create_meta,
    _leaderboard,
    run,
)


def test_available_tasks(capsys):
    args = Namespace(
        categories=None,
        task_types=None,
        languages=None,
        tasks=None,
    )
    _available_tasks(args=args)

    captured = capsys.readouterr()
    assert "LccSentimentClassification" in captured.out, (
        "Sample task LccSentimentClassification task not found in available tasks"
    )


def test_available_benchmarks(capsys):
    args = Namespace(benchmarks=None)
    _available_benchmarks(args=args)

    captured = capsys.readouterr()
    assert "MTEB(eng, v1)" in captured.out, (
        "Sample benchmark MTEB(eng, v1) task not found in available benchmarks"
    )


run_task_fixures = [
    (
        "sentence-transformers/all-MiniLM-L6-v2",
        "BornholmBitextMining",
        "8b3219a92973c328a8e22fadcfa821b5dc75636a",
    ),
]


@pytest.mark.parametrize("model_name,task_name,model_revision", run_task_fixures)
def test_run_task(
    model_name: str,
    task_name: str,
    model_revision: str,
    tmp_path: Path,
):
    args = Namespace(
        model=model_name,
        tasks=[task_name],
        model_revision=model_revision,
        output_folder=tmp_path.as_posix(),
        device=None,
        categories=None,
        task_types=None,
        languages=None,
        batch_size=None,
        verbosity=3,
        co2_tracker=None,
        overwrite_strategy="always",
        eval_splits=None,
        prediction_folder=None,
        benchmarks=None,
        overwrite=False,
        save_predictions=None,
    )

    run(args)

    model_name_as_path = model_name.replace("/", "__").replace(" ", "_")
    results_path = tmp_path / "results" / model_name_as_path / model_revision
    assert results_path.exists(), "Output folder not created"
    assert "model_meta.json" in [f.name for f in list(results_path.glob("*.json"))], (
        "model_meta.json not found in output folder"
    )
    assert f"{task_name}.json" in [f.name for f in list(results_path.glob("*.json"))], (
        f"{task_name} not found in output folder"
    )


def test_create_meta(tmp_path):
    """Test create_meta function directly as well as through the command line interface"""
    test_folder = Path(__file__).parent
    model_name = "sentence-transformers/all-MiniLM-L6-v2"
    output_folder = test_folder / "create_meta"
    output_path = tmp_path / "model_card.md"

    args = Namespace(
        model_name=model_name,
        results_folder=output_folder,
        output_path=output_path,
        overwrite=True,
        from_existing=None,
        tasks=None,
        benchmarks=None,
    )

    _create_meta(args)

    assert output_path.exists(), "Output file not created"

    with output_path.open("r") as f:
        meta = f.read()
        meta = meta[meta.index("---") + 3 : meta.index("---", meta.index("---") + 3)]
        frontmatter = yaml.safe_load(meta)

    with (output_folder / "model_card_gold.md").open("r") as f:
        gold = f.read()
        gold = gold[gold.index("---") + 3 : gold.index("---", gold.index("---") + 3)]
        frontmatter_gold = yaml.safe_load(gold)

    # compare the frontmatter (ignoring the order of keys and other elements)
    for key in frontmatter_gold:
        assert key in frontmatter, f"Key {key} not found in output"

        assert frontmatter[key] == frontmatter_gold[key], (
            f"Value for {key} does not match"
        )

    # ensure that the command line interface works as well
    command = f"{sys.executable} -m mteb create-model-results --model-name {model_name} --results-folder {output_folder.as_posix()} --output-path {output_path.as_posix()} --overwrite"
    result = subprocess.run(command, shell=True, capture_output=True, text=True)
    assert result.returncode == 0, "Command failed"


@pytest.mark.parametrize(
    "existing_readme_name, gold_readme_name",
    [
        ("existing_readme.md", "model_card_gold_existing.md"),
        ("model_card_without_frontmatter.md", "model_card_gold_without_frontmatter.md"),
    ],
)
def test_create_meta_from_existing(
    existing_readme_name: str, gold_readme_name: str, tmp_path: Path
):
    """Test create_meta function directly as well as through the command line interface"""
    test_folder = Path(__file__).parent
    model_name = "sentence-transformers/all-MiniLM-L6-v2"
    output_folder = test_folder / "create_meta"
    output_path = tmp_path / "model_card.md"
    existing_readme = output_folder / existing_readme_name

    args = Namespace(
        model_name=model_name,
        results_folder=output_folder,
        output_path=output_path,
        overwrite=True,
        from_existing=str(existing_readme),
        tasks=None,
        benchmarks=None,
    )

    _create_meta(args)

    assert output_path.exists(), "Output file not created"

    yaml_start_sep = "---"
    yaml_end_sep = "\n---\n"  # newline to avoid matching "---" in the content

    with output_path.open("r") as f:
        meta = f.read()
        start_yaml = meta.index(yaml_start_sep) + len(yaml_start_sep)
        end_yaml = meta.index(yaml_end_sep, start_yaml)
        readme_output = meta[end_yaml + len(yaml_end_sep) :]
        meta = meta[start_yaml:end_yaml]
        frontmatter = yaml.safe_load(meta)

    with (output_folder / gold_readme_name).open("r") as f:
        gold = f.read()
        start_yaml = gold.index(yaml_start_sep) + len(yaml_start_sep)
        end_yaml = gold.index(yaml_end_sep, start_yaml)
        gold_readme = gold[end_yaml + len(yaml_end_sep) :]
        gold = gold[start_yaml:end_yaml]
        frontmatter_gold = yaml.safe_load(gold)

    # compare the frontmatter (ignoring the order of keys and other elements)
    for key in frontmatter_gold:
        assert key in frontmatter, f"Key {key} not found in output"

        assert frontmatter[key] == frontmatter_gold[key], (
            f"Value for {key} does not match"
        )
    assert readme_output == gold_readme
    # ensure that the command line interface works as well
    command = f"{sys.executable} -m mteb create-model-results --model-name {model_name} --results-folder {output_folder.as_posix()} --output-path {output_path.as_posix()} --from-existing {existing_readme.as_posix()} --overwrite"
    result = subprocess.run(command, shell=True, capture_output=True, text=True)
    assert result.returncode == 0, "Command failed"


def test_leaderboard_help():
    """Test that leaderboard help command works."""
    command = [sys.executable, "-m", "mteb", "leaderboard", "--help"]
    result = subprocess.run(command, capture_output=True, text=True)

    assert result.returncode == 0, "Leaderboard help command failed"
    assert "--cache-path" in result.stdout, "--cache-path option not found in help"
    assert "--host" in result.stdout, "--host option not found in help"
    assert "--port" in result.stdout, "--port option not found in help"
    assert "--share" in result.stdout, "--share option not found in help"
    assert "Path to the cache folder containing model results" in result.stdout, (
        "Cache path description not found"
    )


@pytest.mark.parametrize(
    "cache_path_input,host,port,share,test_description",
    [
        ("custom", "localhost", 8080, True, "custom cache path"),
        (None, "127.0.0.1", 7860, False, "default cache path"),
    ],
)
def test_leaderboard_cache_paths(
    tmp_path: Path, cache_path_input, host, port, share, test_description
):
    """Test leaderboard with different cache path configurations."""

    # Set up cache path based on parameter
    if cache_path_input == "custom":
        custom_cache = tmp_path / "my_results"
        custom_cache.mkdir(exist_ok=True)
        cache_path = str(custom_cache)
        expected_cache_path = custom_cache
    else:
        cache_path = None
        from mteb.cache import ResultCache

        expected_cache_path = ResultCache().default_cache_path

    # Mock the get_leaderboard_app function and the gradio app
    mock_app = MagicMock()
    mock_app.launch = MagicMock()

    # Create a mock function that captures the cache argument and returns our mock app
    def mock_get_app_func(cache, rebuild=False):
        # Store the cache for verification
        mock_get_app_func.called_with_cache = cache
        mock_get_app_func.called_with_rebuild = rebuild
        return mock_app

    # Mock gradio themes
    mock_theme = MagicMock()
    mock_font = MagicMock()

    # Patch the local import inside _leaderboard function
    with patch.dict(
        "sys.modules",
        {
            "mteb.leaderboard": MagicMock(get_leaderboard_app=mock_get_app_func),
            "gradio": MagicMock(
                themes=MagicMock(
                    Soft=MagicMock(return_value=mock_theme),
                    GoogleFont=MagicMock(return_value=mock_font),
                )
            ),
        },
    ):
        args = Namespace(
            cache_path=cache_path,
            host=host,
            port=port,
            share=share,
            rebuild=False,
        )

        _leaderboard(args)

        # Verify get_leaderboard_app was called with a cache that has the correct path
        assert hasattr(mock_get_app_func, "called_with_cache"), (
            "get_leaderboard_app was not called"
        )
        cache_instance = mock_get_app_func.called_with_cache
        assert cache_instance.cache_path == expected_cache_path, (
            f"Expected cache path {expected_cache_path}, got {cache_instance.cache_path}"
        )

        # Verify launch parameters
        mock_app.launch.assert_called_once_with(
            server_name=host,
            server_port=port,
            share=share,
            theme=mock_theme,
            head='\n    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">\n    ',
        )


def test_leaderboard_cli_integration():
    """Test the full CLI command integration."""
    # Test that the command is recognized by the CLI
    command = [sys.executable, "-m", "mteb", "--help"]
    result = subprocess.run(command, capture_output=True, text=True)

    assert result.returncode == 0
    assert "leaderboard" in result.stdout, "Leaderboard command not found in main help"



"""Perplexity AI embedding models."""

from __future__ import annotations

from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

langs = [
    "afr-Latn",
    "ara-Arab",
    "aze-Latn",
    "bel-Cyrl",
    "bul-Cyrl",
    "ben-Beng",
    "cat-Latn",
    "ceb-Latn",
    "ces-Latn",
    "cym-Latn",
    "dan-Latn",
    "deu-Latn",
    "ell-Grek",
    "eng-Latn",
    "spa-Latn",
    "est-Latn",
    "eus-Latn",
    "fas-Arab",
    "fin-Latn",
    "fra-Latn",
    "glg-Latn",
    "guj-Gujr",
    "heb-Hebr",
    "hin-Deva",
    "hrv-Latn",
    "hat-Latn",
    "hun-Latn",
    "hye-Armn",
    "ind-Latn",
    "isl-Latn",
    "ita-Latn",
    "jpn-Jpan",
    "jav-Latn",
    "kat-Geor",
    "kaz-Cyrl",
    "khm-Khmr",
    "kan-Knda",
    "kor-Hang",
    "kir-Cyrl",
    "lao-Laoo",
    "lit-Latn",
    "lav-Latn",
    "mkd-Cyrl",
    "mal-Mlym",
    "mon-Cyrl",
    "mar-Deva",
    "msa-Latn",
    "mya-Mymr",
    "nep-Deva",
    "nld-Latn",
    "nor-Latn",
    "nob-Latn",
    "nno-Latn",
    "pan-Guru",
    "pol-Latn",
    "por-Latn",
    "que-Latn",
    "ron-Latn",
    "rus-Cyrl",
    "sin-Sinh",
    "slk-Latn",
    "slv-Latn",
    "swa-Latn",
    "tam-Taml",
    "tel-Telu",
    "tha-Thai",
    "tgl-Latn",
    "tur-Latn",
    "ukr-Cyrl",
    "urd-Arab",
    "vie-Latn",
    "yor-Latn",
    "zho-Hans",
]

PERPLEXITY_CITATION = """@misc{eslami2026diffusionpretraineddensecontextualembeddings,
      title={Diffusion-Pretrained Dense and Contextual Embeddings},
      author={Sedigheh Eslami and Maksim Gaiduk and Markus Krimmel and Louis Milliken and Bo Wang and Denis Bykov},
      year={2026},
      eprint={2602.11151},
      archivePrefix={arXiv},
      primaryClass={cs.LG},
      url={https://arxiv.org/abs/2602.11151},
}"""

pplx_embed_v1_0_6b = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="perplexity-ai/pplx-embed-v1-0.6b",
    loader_kwargs={"trust_remote_code": True},
    model_type=["dense"],
    languages=langs,
    open_weights=True,
    revision="1dc2ea99a948a2f17b103949ad02b0194a20c0a8",
    release_date="2026-02-11",
    n_parameters=596_049_920,
    n_embedding_parameters=155_582_464,
    memory_usage_mb=2274,
    embed_dim=1024,
    license="mit",
    max_tokens=32768,
    reference="https://huggingface.co/perplexity-ai/pplx-embed-v1-0.6b",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="Qwen/Qwen3-0.6B",
    training_datasets={
        "MSMARCO",
        "MIRACLRetrieval",
        "NQ",
        "FiQA2018",
        "FEVER",
        "HotpotQA",
    },
    public_training_code=None,
    public_training_data=None,
    citation=PERPLEXITY_CITATION,
)

pplx_embed_v1_4b = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="perplexity-ai/pplx-embed-v1-4b",
    loader_kwargs={"trust_remote_code": True},
    model_type=["dense"],
    languages=langs,
    open_weights=True,
    revision="964a2aafe021f2bbda88ba17eb326eb849922435",
    release_date="2026-02-11",
    n_parameters=4_000_000_000,
    n_embedding_parameters=388_956_160,
    memory_usage_mb=15344,
    embed_dim=2560,
    license="mit",
    max_tokens=32768,
    reference="https://huggingface.co/perplexity-ai/pplx-embed-v1-4b",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="Qwen/Qwen3-4B",
    training_datasets={
        "MSMARCO",
        "MIRACLRetrieval",
        "NQ",
        "FiQA2018",
        "FEVER",
        "HotpotQA",
    },
    public_training_code=None,
    public_training_data=None,
    citation=PERPLEXITY_CITATION,
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

BLIP2_CITATION = """@inproceedings{li2023blip2,
    title={{BLIP-2:} Bootstrapping Language-Image Pre-training with Frozen Image Encoders and Large Language Models},
    author={Junnan Li and Dongxu Li and Silvio Savarese and Steven Hoi},
    year={2023},
    booktitle={ICML},
}"""


def blip2_loader(model_name, **kwargs):
    requires_package(
        blip2_loader, "salesforce-lavis", model_name, "pip install 'mteb[blip2]'"
    )
    from lavis.models.blip2_models.blip2_image_text_matching import (
        Blip2ITM,
    )

    class BLIP2Model(AbsEncoder):
        def __init__(
            self,
            model_name: str,
            revision: str,
            device: str = "cuda" if torch.cuda.is_available() else "cpu",
            **kwargs: Any,
        ):
            from transformers import Blip2Processor

            self.model_name = model_name
            self.device = device
            model_type = "coco" if "coco" in model_name else "pretrain"
            self.model = (
                Blip2ITM.from_pretrained(model_type, revision=revision)
                .to(self.device)
                .float()
            )
            self.processor = Blip2Processor.from_pretrained(
                model_name, revision=revision
            )

        def get_text_embeddings(
            self,
            texts: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_text_embeddings = []

            with torch.no_grad():
                for batch in tqdm(
                    texts, disable=not show_progress_bar, desc="Text Encoding"
                ):
                    text_tokens = self.model.tokenizer(
                        batch["text"],
                        padding="max_length",
                        truncation=True,
                        max_length=self.model.max_txt_len,
                        return_tensors="pt",
                    ).to(self.device)
                    text_outputs = self.model.forward_text(text_tokens)
                    all_text_embeddings.append(text_outputs.cpu())
            all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
            return all_text_embeddings

        def get_image_embeddings(
            self,
            images: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_image_embeddings = []

            with torch.no_grad():
                for batch in tqdm(
                    images, disable=not show_progress_bar, desc="Image Encoding"
                ):
                    inputs = self.processor(
                        images=batch["image"], return_tensors="pt", padding=True
                    )
                    image_outputs = self.model.forward_image(
                        inputs["pixel_values"].to(self.device)
                    )
                    image_outputs = image_outputs[0][:, 0, :]
                    all_image_embeddings.append(image_outputs.cpu())

            all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
            return all_image_embeddings

        def get_multimodal_embeddings(
            self,
            inputs: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_multimodal_embeddings = []

            with torch.no_grad():
                # check dataloader batch size is the same as batch size
                for batch in tqdm(
                    inputs, disable=not show_progress_bar, desc="Multimodal Encoding"
                ):
                    image_inputs = self.processor(
                        images=batch["image"], return_tensors="pt", padding=True
                    )["pixel_values"].to(self.device)
                    multimodal_outputs = self.model.extract_features(
                        {"text_input": batch["text"], "image": image_inputs}
                    ).multimodal_embeds[:, 0, :]
                    all_multimodal_embeddings.append(multimodal_outputs.cpu())

            return torch.cat(all_multimodal_embeddings, dim=0)

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            **kwargs: Any,
        ) -> Array:
            # TODO: find out if BLIP has a prescribed way of fusing text and image embeddings
            text_embeddings = None
            image_embeddings = None

            if "text" in inputs.dataset.features and "image" in inputs.dataset.features:
                return self.get_multimodal_embeddings(inputs)

            if "text" in inputs.dataset.features:
                text_embeddings = self.get_text_embeddings(inputs, **kwargs)
            if "image" in inputs.dataset.features:
                image_embeddings = self.get_image_embeddings(inputs, **kwargs)

            if text_embeddings is not None and image_embeddings is not None:
                if len(text_embeddings) != len(image_embeddings):
                    raise ValueError(
                        "The number of texts and images must have the same length"
                    )
                fused_embeddings = text_embeddings + image_embeddings
                return fused_embeddings
            elif text_embeddings is not None:
                return text_embeddings
            elif image_embeddings is not None:
                return image_embeddings
            raise ValueError

    return BLIP2Model(model_name, **kwargs)


blip2_training_datasets = set(
    # COCO
    # CC3M+CC12M+SBU
    # LAION400M
)

blip2_opt_2_7b = ModelMeta(
    loader=blip2_loader,
    name="Salesforce/blip2-opt-2.7b",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="51572668da0eb669e01a189dc22abe6088589a24",
    release_date="2024-03-22",
    modalities=["image", "text"],
    n_parameters=3_740_000_000,
    n_embedding_parameters=128696320,
    memory_usage_mb=14285,
    max_tokens=None,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/salesforce/LAVIS/tree/main/projects/blip2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/Salesforce/blip2-opt-2.7b",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=blip2_training_datasets,
    citation=BLIP2_CITATION,
)

blip2_opt_6_7b_coco = ModelMeta(
    loader=blip2_loader,
    name="Salesforce/blip2-opt-6.7b-coco",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="0d580de59320a25a4d2c386387bcef310d5f286e",
    release_date="2024-03-31",
    modalities=["image", "text"],
    n_parameters=7_750_000_000,
    n_embedding_parameters=205914112,
    memory_usage_mb=29577,
    max_tokens=None,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/salesforce/LAVIS/tree/main/projects/blip2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/Salesforce/blip2-opt-6.7b-coco",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=blip2_training_datasets,
    citation=BLIP2_CITATION,
)



"""Utilities for setting seeds for reproducibility.

Derived from `transformers.trainer_utils.set_seed`. It assumes torch is installed.
"""

import logging
import random

import numpy as np
import torch

logger = logging.getLogger(__name__)


def _set_seed(seed: int) -> tuple[random.Random, np.random.Generator]:
    """Helper function for reproducible behavior to set the seed in `random`, `numpy`, `torch` and/or `tf` (if installed).

    If does not set the seed for npu, musa, mlu, hpu, xpu devices. To set the seed for those devices, we recommend using:

    ```python
    from transformers import set_seed
    set_seed(seed)
    ```

    Args:
        seed: The seed to set.

    Returns:
        A tuple of (random.Random, np.random.Generator) initialized with the given seed.
    """
    random.seed(seed)
    np.random.seed(seed)  # noqa: NPY002
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    # ^^ safe to call this function even if cuda is not available

    try:
        import tensorflow as tf

        tf.random.set_seed(seed)
    except ImportError:
        pass  # not installed

    return random.Random(seed), np.random.default_rng(seed)



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

embedding_gemma_300m_scandi = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="emillykkejensen/EmbeddingGemma-Scandi-300m",
    model_type=["dense"],
    languages=["dan-Latn", "swe-Latn", "nor-Latn", "nob-Latn", "nno-Latn"],
    open_weights=True,
    revision="9f3307b9f601db564a9190cb475324d128dcfe86",
    release_date="2025-10-17",
    n_parameters=307_581_696,
    n_embedding_parameters=201326592,
    embed_dim=768,
    max_tokens=2048,
    license="apache-2.0",
    reference="https://huggingface.co/emillykkejensen/EmbeddingGemma-Scandi-300m",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/DDSC/nordic-embedding-training-data",
    training_datasets=set(),
    similarity_fn_name="cosine",  # type: ignore[arg-type]
    adapted_from="google/embeddinggemma-300m",
    memory_usage_mb=578,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)


qwen_scandi = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="emillykkejensen/Qwen3-Embedding-Scandi-0.6B",
    model_type=["dense"],
    languages=["dan-Latn", "swe-Latn", "nor-Latn", "nob-Latn", "nno-Latn"],
    open_weights=True,
    revision="cf1e7ba36ebd3d605549d8f02930a18e17b54513",
    release_date="2025-10-17",
    n_parameters=595776512,
    n_embedding_parameters=155309056,
    memory_usage_mb=2272,
    embed_dim=1024,
    max_tokens=32768,
    license="apache-2.0",
    reference="https://huggingface.co/emillykkejensen/Qwen3-Embedding-Scandi-0.6B",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/DDSC/nordic-embedding-training-data",
    training_datasets=set(),
    similarity_fn_name="cosine",  # type: ignore[arg-type]
    adapted_from="Qwen/Qwen3-Embedding-0.6B",
)


mmbert_scandi = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="emillykkejensen/mmBERTscandi-base-embedding",
    model_type=["dense"],
    languages=["dan-Latn", "swe-Latn", "nor-Latn", "nob-Latn", "nno-Latn"],
    open_weights=True,
    revision="82d74c7a5d8e1ddf31b132865df2d16b2b0294ee",
    release_date="2025-10-17",
    n_parameters=306939648,
    n_embedding_parameters=196608000,
    memory_usage_mb=1171,
    embed_dim=768,
    max_tokens=8192,
    license="apache-2.0",
    reference="https://huggingface.co/emillykkejensen/Qwen3-Embedding-Scandi-0.6B",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/DDSC/nordic-embedding-training-data",
    training_datasets=set(),
    similarity_fn_name="cosine",  # type: ignore[arg-type]
    adapted_from="jonasaise/scandmmBERT-base-scandinavian",
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm
from transformers import ClapModel, ClapProcessor
from transformers.modeling_outputs import BaseModelOutputWithPooling

from mteb._create_dataloaders import AudioCollator
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput, TextInput


class ClapZeroShotWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        self.model_name = model_name
        self.device = device
        self.model = ClapModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.processor = ClapProcessor.from_pretrained(model_name, revision=revision)
        # CLAP's expected sampling rate. If the input audio is not sampled at this rate,
        # it will raise a ValueError similar to: ValueError: The model corresponding to
        # this feature extractor: ClapFeatureExtractor was trained using a sampling rate
        # of 48000. Please make sure that the provided `raw_speech` input was sampled
        # with 48000 and not 44100.
        self.sampling_rate = self.processor.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> np.ndarray:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_features = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_array = [audio["array"] for audio in batch["audio"]]
            features = self.processor(
                audio=audio_array,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding=True,
            )
            features = {k: v.to(self.device) for k, v in features.items()}

            with torch.no_grad():
                audio_features = self.model.get_audio_features(**features)
                if isinstance(audio_features, BaseModelOutputWithPooling):
                    audio_features = audio_features.pooler_output
                audio_features = audio_features / audio_features.norm(
                    dim=-1, keepdim=True
                )
                all_features.append(audio_features.cpu().detach().numpy())

        return np.vstack(all_features)

    def get_text_embeddings(
        self,
        inputs: DataLoader[TextInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = []
        for batch in tqdm(
            inputs, disable=not show_progress_bar, desc="Processing text batches"
        ):
            texts = batch["text"]
            inputs = self.processor(
                text=texts,
                return_tensors="pt",
                padding=True,
                truncation=True,
            )
            inputs = {k: v.to(self.device) for k, v in inputs.items()}

            text_features = self.model.get_text_features(**inputs)
            # Normalize embeddings
            text_features = text_features / text_features.norm(dim=-1, keepdim=True)
            text_embeddings.append(text_features.cpu().detach().numpy())

        return np.vstack(text_embeddings)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        audio_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "audio" in inputs.dataset.features:
            audio_embeddings = self.get_audio_embeddings(inputs, **kwargs)

        if text_embeddings is not None and audio_embeddings is not None:
            if len(text_embeddings) != len(audio_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + audio_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif audio_embeddings is not None:
            return audio_embeddings
        raise ValueError


# Model metadata
clap_htsat_fused = ModelMeta(
    loader=ClapZeroShotWrapper,
    name="laion/clap-htsat-fused",
    languages=["eng-Latn"],
    revision="cca9e288ab447cee67d9ada1f85ddb46500f1401",
    release_date="2023-05-22",
    modalities=["audio", "text"],
    n_parameters=153_507_530,
    n_embedding_parameters=38603520,
    memory_usage_mb=586,  # Calculated using model.calculate_memory_usage_mb()
    max_tokens=float("inf"),
    embed_dim=512,  # The project_dim in config.json is 512
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/LAION-AI/CLAP",
    public_training_data="https://laion.ai/blog/laion-audio-630k/",
    framework=["PyTorch"],
    reference="https://huggingface.co/laion/clap-htsat-fused",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(
        # "LAION-Audio-630K": ["https://laion.ai/blog/laion-audio-630k/"]
    ),
    citation="""
@misc{wu2024largescalecontrastivelanguageaudiopretraining,
      title={Large-scale Contrastive Language-Audio Pretraining with Feature Fusion and Keyword-to-Caption Augmentation},
      author={Yusong Wu and Ke Chen and Tianyu Zhang and Yuchen Hui and Marianna Nezhurina and Taylor Berg-Kirkpatrick and Shlomo Dubnov},
      year={2024},
      eprint={2211.06687},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2211.06687},
}
""",
)


clap_htsat_unfused = ModelMeta(
    loader=ClapZeroShotWrapper,
    name="laion/clap-htsat-unfused",
    languages=["eng-Latn"],
    revision="8fa0f1c6d0433df6e97c127f64b2a1d6c0dcda8a",
    release_date="2023-05-22",
    modalities=["audio", "text"],
    n_parameters=153_492_890,
    n_embedding_parameters=38603520,
    memory_usage_mb=586,  # Calculated using model.calculate_memory_usage_mb()
    max_tokens=float("inf"),
    embed_dim=512,  # The project_dim in config.json is 512
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/LAION-AI/CLAP",
    public_training_data="https://laion.ai/blog/laion-audio-630k/",
    framework=["PyTorch"],
    reference="https://huggingface.co/laion/clap-htsat-unfused",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(
        # "LAION-Audio-630K": ["https://laion.ai/blog/laion-audio-630k/"]
    ),
    citation="""
@misc{wu2024largescalecontrastivelanguageaudiopretraining,
      title={Large-scale Contrastive Language-Audio Pretraining with Feature Fusion and Keyword-to-Caption Augmentation},
      author={Yusong Wu and Ke Chen and Tianyu Zhang and Yuchen Hui and Marianna Nezhurina and Taylor Berg-Kirkpatrick and Shlomo Dubnov},
      year={2024},
      eprint={2211.06687},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2211.06687},
}
""",
)

larger_clap_general = ModelMeta(
    loader=ClapZeroShotWrapper,
    name="laion/larger_clap_general",
    languages=["eng-Latn"],
    revision="ada0c23a36c4e8582805bb38fec3905903f18b41",
    release_date="2023-05-22",
    modalities=["audio", "text"],
    n_parameters=193_913_882,  # Calculated using torch.numel(model.parameters())
    memory_usage_mb=740,  # Calculated using model.calculate_memory_usage_mb()
    max_tokens=float("inf"),
    embed_dim=512,  # The project_dim (for even larger clap general) in config.json is 512
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/LAION-AI/CLAP",
    public_training_data="https://laion.ai/blog/laion-audio-630k/",
    framework=["PyTorch"],
    reference="https://huggingface.co/laion/larger_clap_general",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(
        # "LAION-Audio-630K": ["https://laion.ai/blog/laion-audio-630k/"]
    ),  # Additional finetuning over music dataset but not specified what the exact dataset is
    citation="""
@misc{wu2024largescalecontrastivelanguageaudiopretraining,
      title={Large-scale Contrastive Language-Audio Pretraining with Feature Fusion and Keyword-to-Caption Augmentation},
      author={Yusong Wu and Ke Chen and Tianyu Zhang and Yuchen Hui and Marianna Nezhurina and Taylor Berg-Kirkpatrick and Shlomo Dubnov},
      year={2024},
      eprint={2211.06687},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2211.06687},
}
""",
)

larger_clap_music = ModelMeta(
    loader=ClapZeroShotWrapper,
    name="laion/larger_clap_music",
    languages=["eng-Latn"],
    revision="a0b4534a14f58e20944452dff00a22a06ce629d1",
    release_date="2023-05-22",
    modalities=["audio", "text"],
    n_parameters=193_913_882,  # Calculated using torch.numel(model.parameters())
    memory_usage_mb=740,  # Calculated using model.calculate_memory_usage_mb()
    max_tokens=float("inf"),
    embed_dim=512,  # The project_dim (for even larger clap general) in config.json is 512
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/LAION-AI/CLAP",
    public_training_data="https://laion.ai/blog/laion-audio-630k/",
    framework=["PyTorch"],
    reference="https://huggingface.co/laion/larger_clap_music",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(
        # "LAION-Audio-630K": ["https://laion.ai/blog/laion-audio-630k/"]
    ),  # Additional finetuning over music dataset but not specified what the exact dataset is
    citation="""
@misc{wu2024largescalecontrastivelanguageaudiopretraining,
      title={Large-scale Contrastive Language-Audio Pretraining with Feature Fusion and Keyword-to-Caption Augmentation},
      author={Yusong Wu and Ke Chen and Tianyu Zhang and Yuchen Hui and Marianna Nezhurina and Taylor Berg-Kirkpatrick and Shlomo Dubnov},
      year={2024},
      eprint={2211.06687},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2211.06687},
}
""",
)

larger_clap_music_and_speech = ModelMeta(
    loader=ClapZeroShotWrapper,
    name="laion/larger_clap_music_and_speech",
    languages=["eng-Latn"],
    revision="195c3a3e68faebb3e2088b9a79e79b43ddbda76b",
    release_date="2023-05-22",
    modalities=["audio", "text"],
    n_parameters=193_913_882,  # Calculated using torch.numel(model.parameters())
    memory_usage_mb=740,  # Calculated using model.calculate_memory_usage_mb()
    max_tokens=float("inf"),
    embed_dim=512,  # The project_dim (for even larger clap general) in config.json is 512
    license="mit",
    open_weights=True,
    public_training_code="https://github.com/LAION-AI/CLAP",
    public_training_data="https://laion.ai/blog/laion-audio-630k/",
    framework=["PyTorch"],
    reference="https://huggingface.co/laion/larger_clap_music_and_speech",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(
        # "LAION-Audio-630K": ["https://laion.ai/blog/laion-audio-630k/"]
    ),  # Additional finetuning over music dataset but not specified what the exact dataset is
    citation="""
@misc{wu2024largescalecontrastivelanguageaudiopretraining,
      title={Large-scale Contrastive Language-Audio Pretraining with Feature Fusion and Keyword-to-Caption Augmentation},
      author={Yusong Wu and Ke Chen and Tianyu Zhang and Yuchen Hui and Marianna Nezhurina and Taylor Berg-Kirkpatrick and Shlomo Dubnov},
      year={2024},
      eprint={2211.06687},
      archivePrefix={arXiv},
      primaryClass={cs.SD},
      url={https://arxiv.org/abs/2211.06687},
}
""",
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import (
    requires_image_dependencies,
    requires_package,
)
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from PIL import Image
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

logger = logging.getLogger(__name__)


class ColPaliEngineWrapper(AbsEncoder):
    """Base wrapper for `colpali_engine` models. Adapted from https://github.com/illuin-tech/colpali/tree/bebcdd6715dba42624acd8d7f7222a16a5daf848/colpali_engine/models"""

    def __init__(
        self,
        model_name: str,
        model_class: type,
        processor_class: type,
        revision: str | None = None,
        device: str | None = None,
        **kwargs,
    ):
        requires_image_dependencies()
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )

        self.device = device or ("cuda" if torch.cuda.is_available() else "cpu")

        # Load model
        self.mdl = model_class.from_pretrained(
            model_name,
            device_map=self.device,
            adapter_kwargs={"revision": revision},
            **kwargs,
        )
        self.mdl.eval()

        # Load processor
        self.processor = processor_class.from_pretrained(model_name)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError

    def encode_input(self, inputs):
        return self.mdl(**inputs)

    def get_image_embeddings(
        self,
        images,
        batch_size: int = 32,
        **kwargs,
    ):
        import torchvision.transforms.functional as F
        from PIL import Image

        all_embeds = []

        with torch.no_grad():
            for batch in tqdm(images, desc="Encoding images"):
                # batch may be list of tensors or PIL
                imgs = [
                    F.to_pil_image(b.to(self.device))
                    if not isinstance(b, Image.Image)
                    else b
                    for b in batch["image"]
                ]
                inputs = self.processor.process_images(imgs)
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self.encode_input(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def get_text_embeddings(
        self,
        texts,
        batch_size: int = 32,
        **kwargs,
    ):
        all_embeds = []
        with torch.no_grad():
            for batch in tqdm(texts, desc="Encoding texts"):
                batch = [
                    self.processor.query_prefix
                    + t
                    + self.processor.query_augmentation_token * 10
                    for t in batch["text"]
                ]
                inputs = self.processor.process_queries(batch)
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self.encode_input(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def get_fused_embeddings(
        self,
        texts: list[str] | None = None,
        images: list[Image.Image] | DataLoader | None = None,
        *,
        task_name: str | None = None,
        prompt_type: PromptType | None = None,
        batch_size: int = 32,
        fusion_mode="sum",
        **kwargs: Any,
    ):
        raise NotImplementedError(
            "Fused embeddings are not supported yet. Please use get_text_embeddings or get_image_embeddings."
        )

    def similarity(self, a, b):
        return self.processor.score(a, b, device=self.device)


class ColPaliWrapper(ColPaliEngineWrapper):
    """Wrapper for ColPali models."""

    def __init__(
        self,
        model_name: str = "vidore/colpali-v1.3",
        revision: str | None = None,
        device: str | None = None,
        **kwargs,
    ):
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )
        from colpali_engine.models import ColPali, ColPaliProcessor

        super().__init__(
            model_name=model_name,
            model_class=ColPali,
            processor_class=ColPaliProcessor,
            revision=revision,
            device=device,
            **kwargs,
        )


COLPALI_CITATION = """
@misc{faysse2024colpali,
  title={ColPali: Efficient Document Retrieval with Vision Language Models},
  author={Faysse, Manuel and Sibille, Hugues and Wu, Tony and Omrani, Bilel and Viaud, Gautier and Hudelot, C\'eline and Colombo, Pierre},
  year={2024},
  eprint={2407.01449},
  archivePrefix={arXiv},
  primaryClass={cs.IR}
}"""

COLPALI_TRAINING_DATA = {
    # from https://huggingface.co/datasets/vidore/colpali_train_set
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
}

colpali_v1_1 = ModelMeta(
    loader=ColPaliWrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="vidore/colpali-v1.1",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="a0f15e3bcf97110e7ac1bb4be4bcd30eeb31992a",
    release_date="2024-08-21",
    modalities=["image", "text"],
    n_parameters=2_920_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=4700,
    max_tokens=16384,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/vidore/colpali-v1.1",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    training_datasets=COLPALI_TRAINING_DATA,
    citation=COLPALI_CITATION,
)

colpali_v1_2 = ModelMeta(
    loader=ColPaliWrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="vidore/colpali-v1.2",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="6b89bc63c16809af4d111bfe412e2ac6bc3c9451",
    release_date="2024-08-26",
    modalities=["image", "text"],
    n_parameters=2_920_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=4700,
    max_tokens=16384,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/vidore/colpali-v1.2",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    training_datasets=COLPALI_TRAINING_DATA,
    citation=COLPALI_CITATION,
)

colpali_v1_3 = ModelMeta(
    loader=ColPaliWrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="vidore/colpali-v1.3",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="1b5c8929330df1a66de441a9b5409a878f0de5b0",
    release_date="2024-11-01",
    modalities=["image", "text"],
    n_parameters=2_920_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=4700,
    max_tokens=16384,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/vidore/colpali-v1.3",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    training_datasets=COLPALI_TRAINING_DATA,
    citation=COLPALI_CITATION,
)



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class Caltech101ZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="Caltech101ZeroShot",
        description="Classifying images of 101 widely varied objects.",
        reference="https://ieeexplore.ieee.org/document/1384978",
        dataset={
            "path": "mteb/Caltech101",
            "revision": "011e51e5fb01f0c820824734edb7a539ab8e6650",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2003-01-01",
            "2004-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{1384978,
  author = {Li Fei-Fei and Fergus, R. and Perona, P.},
  booktitle = {2004 Conference on Computer Vision and Pattern Recognition Workshop},
  doi = {10.1109/CVPR.2004.383},
  keywords = {Bayesian methods;Testing;Humans;Maximum likelihood estimation;Assembly;Shape;Machine vision;Image recognition;Parameter estimation;Image databases},
  number = {},
  pages = {178-178},
  title = {Learning Generative Visual Models from Few Training Examples: An Incremental Bayesian Approach Tested on 101 Object Categories},
  volume = {},
  year = {2004},
}
""",
    )

    def get_candidate_labels(self) -> list[str]:
        return [
            f"a photo of a {name}."
            for name in self.dataset["test"].features[self.label_column_name].names
        ]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MrTyDiJaRetrievalLite(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MrTyDiJaRetrievalLite",
        dataset={
            "path": "mteb/MrTyDiJaRetrievalLite",
            "revision": "b87e6ff25f4e32d1c97498a539ea8aad5fde3cb1",
        },
        description=(
            "Mr.TyDi is a multilingual benchmark dataset built on TyDi for document retrieval tasks. "
            "This is the lightweight Japanese version with a reduced corpus (93,382 documents) constructed using "
            "hard negatives from 5 high-performance models."
        ),
        reference="https://huggingface.co/datasets/castorini/mr-tydi",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2025-01-01"),
        domains=["Encyclopaedic", "Non-fiction", "Written"],
        task_subtypes=["Question answering"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        adapted_from=["MrTidyRetrieval"],
        bibtex_citation=r"""
@misc{jmteb_lite,
  author = {Li, Shengzhe and Ohagi, Masaya and Ri, Ryokan and Fukuchi, Akihiko and Shibata, Tomohide
and Kawahara, Daisuke},
  howpublished = {\url{https://huggingface.co/datasets/sbintuitions/JMTEB-lite}},
  title = {{J}{M}{T}{E}{B}-lite: {T}he {L}ightweight {V}ersion of {JMTEB}},
  year = {2025},
}

@article{mrtydi,
  author = {Xinyu Zhang and Xueguang Ma and Peng Shi and Jimmy Lin},
  journal = {arXiv:2108.08787},
  title = {{Mr. TyDi}: A Multi-lingual Benchmark for Dense Retrieval},
  year = {2021},
}
""",
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

SIGLIP_CITATION = """@misc{zhai2023sigmoid,
      title={Sigmoid Loss for Language Image Pre-Training},
      author={Xiaohua Zhai and Basil Mustafa and Alexander Kolesnikov and Lucas Beyer},
      year={2023},
      eprint={2303.15343},
      archivePrefix={arXiv},
      primaryClass={cs.CV}
}"""


class SiglipModelWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import AutoModel, AutoProcessor

        # for sentencepiece and protobuf:
        requires_package(
            self, "sentencepiece", model_name, "pip install 'mteb[siglip]'"
        )

        self.model_name = model_name
        self.device = device
        self.model = AutoModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.processor = AutoProcessor.from_pretrained(model_name, revision=revision)

    def get_text_embeddings(
        self,
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_text_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                inputs = self.processor(
                    text=batch["text"],
                    return_tensors="pt",
                    padding="max_length",
                    truncation=True,
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                text_outputs = self.model.get_text_features(**inputs)
                embeddings = text_outputs.pooler_output
                all_text_embeddings.append(embeddings.cpu())

        all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
        return all_text_embeddings

    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_image_embeddings = []

        with torch.no_grad():
            for batch in tqdm(images):
                _images = [img.convert("RGB") for img in batch["image"]]
                inputs = self.processor(
                    images=_images, return_tensors="pt", padding=True
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                image_outputs = self.model.get_image_features(**inputs)
                embeddings = image_outputs.pooler_output
                all_image_embeddings.append(embeddings.cpu())
        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


siglip_training_datasets = set(
    # WebLI https://arxiv.org/abs/2209.06794
)

siglip_so400m_patch14_224 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-so400m-patch14-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="d04cf29fca7b6374f74d8bea1969314492266b5e",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=877360306,
    n_embedding_parameters=36864000,
    memory_usage_mb=3347,
    max_tokens=16,
    embed_dim=1152,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-so400m-patch14-224",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_so400m_patch14_384 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-so400m-patch14-384",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="9fdffc58afc957d1a03a25b10dba0329ab15c2a3",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=877960498,
    n_embedding_parameters=36864000,
    memory_usage_mb=3349,
    max_tokens=64,
    embed_dim=1152,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-so400m-patch14-384",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_so400m_patch16_256_i18n = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-so400m-patch16-256-i18n",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="365d321c0cfdea96bc28e3a29787a11a062681a1",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=1128758962,
    n_embedding_parameters=288000000,
    memory_usage_mb=4306,
    max_tokens=64,
    embed_dim=1152,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-so400m-patch16-256-i18n",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_base_patch16_256_multilingual = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-base-patch16-256-multilingual",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="8952a4eafcde3cb7ab46b1dd629b33f8784ca9c6",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=370626050,
    n_embedding_parameters=192000000,
    memory_usage_mb=1414,
    max_tokens=64,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-base-patch16-256-multilingual",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_base_patch16_256 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-base-patch16-256",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="b078df89e446d623010d890864d4207fe6399f61",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=203202050,
    n_embedding_parameters=24576000,
    memory_usage_mb=775,
    max_tokens=64,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-base-patch16-256",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_base_patch16_512 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-base-patch16-512",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="753a949581523b60257d93e18391e8c27f72eb22",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=203791874,
    n_embedding_parameters=24576000,
    memory_usage_mb=777,
    max_tokens=64,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-base-patch16-512",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_base_patch16_384 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-base-patch16-384",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="41aec1c83b32e0a6fca20ad88ba058aa5b5ea394",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=203447810,
    n_embedding_parameters=24576000,
    memory_usage_mb=776,
    max_tokens=64,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-base-patch16-384",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_base_patch16_224 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-base-patch16-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="7fd15f0689c79d79e38b1c2e2e2370a7bf2761ed",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=203155970,
    n_embedding_parameters=24576000,
    memory_usage_mb=775,
    max_tokens=64,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-base-patch16-224",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_large_patch16_256 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-large-patch16-256",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="d0da9f876e7d66b4e250cd2450c3ba2ce735e447",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=652150786,
    n_embedding_parameters=32768000,
    memory_usage_mb=2488,
    max_tokens=64,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-large-patch16-256",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)

siglip_large_patch16_384 = ModelMeta(
    loader=SiglipModelWrapper,
    name="google/siglip-large-patch16-384",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="ce005573a40965dfd21fd937fbdeeebf2439fc35",
    release_date="2024-01-08",
    modalities=["image", "text"],
    n_parameters=652478466,
    n_embedding_parameters=32768000,
    memory_usage_mb=2489,
    max_tokens=64,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/google-research/big_vision/blob/main/big_vision/trainers/proj/image_text/siglip.py",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/google/siglip-large-patch16-384",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=siglip_training_datasets,
    citation=SIGLIP_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class JaCWIRRetrievalLite(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JaCWIRRetrievalLite",
        dataset={
            "path": "mteb/JaCWIRRetrievalLite",
            "revision": "79472b360242cf2692e24a6d9999ef50d350d672",
        },
        description=(
            "JaCWIR (Japanese Casual Web IR) is a dataset consisting of questions and webpage meta descriptions "
            "collected from Hatena Bookmark. This is the lightweight version with a reduced corpus "
            "(302,638 documents) constructed using hard negatives from 5 high-performance models."
        ),
        reference="https://huggingface.co/datasets/hotchpotch/JaCWIR",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2025-01-01"),
        domains=["Web", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        adapted_from=["JaCWIRRetrieval"],
        bibtex_citation=r"""
@misc{jmteb_lite,
  author = {Li, Shengzhe and Ohagi, Masaya and Ri, Ryokan and Fukuchi, Akihiko and Shibata, Tomohide
and Kawahara, Daisuke},
  howpublished = {\url{https://huggingface.co/datasets/sbintuitions/JMTEB-lite}},
  title = {{J}{M}{T}{E}{B}-lite: {T}he {L}ightweight {V}ersion of {JMTEB}},
  year = {2025},
}

@misc{yuichi-tateno-2024-jacwir,
  author = {Yuichi Tateno},
  title = {JaCWIR: Japanese Casual Web IR - 日本語情報検索評価のための小規模でカジュアルなWebタイトルと概要のデータセット},
  url = {https://huggingface.co/datasets/hotchpotch/JaCWIR},
}
""",
    )



from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class VABBClusteringP2P(AbsTaskClustering):
    max_fraction_of_documents_to_embed = 1.0
    metadata = TaskMetadata(
        name="VABBClusteringP2P",
        dataset={
            "path": "clips/mteb-nl-vabb-cls",
            "revision": "544acc2e46909eab2b49962b043a18b9c9772770",
        },
        description="This dataset contains the fourteenth edition of the Flemish Academic Bibliography for the Social "
        "Sciences and Humanities (VABB-SHW), a database of academic publications from the social sciences "
        "and humanities authored by researchers affiliated to Flemish universities (more information). "
        "Publications in the database are used as one of the parameters of the Flemish performance-based "
        "research funding system",
        reference="https://zenodo.org/records/14214806",
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="v_measure",
        date=("2009-11-01", "2010-01-01"),
        domains=["Academic", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@dataset{aspeslagh2024vabb,
  author = {Aspeslagh, Pieter and Guns, Raf and Engels, Tim C. E.},
  doi = {10.5281/zenodo.14214806},
  publisher = {Zenodo},
  title = {VABB-SHW: Dataset of Flemish Academic Bibliography for the Social Sciences and Humanities (edition 14)},
  url = {https://doi.org/10.5281/zenodo.14214806},
  year = {2024},
}
""",
        prompt="Identificeer de hoofdcategorie van wetenschappelijke artikelen op basis van de titels en abstracts",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        for split in self.dataset:
            self.dataset[split] = self.dataset[split].map(
                lambda ex: {
                    "labels": ex["org_discipline"],
                    "sentences": f"{ex['title']}\n{ex['abstract']}",
                }
            )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class DutchColaClassification(AbsTaskClassification):
    samples_per_label = 128
    metadata = TaskMetadata(
        name="DutchColaClassification",
        description="Dutch CoLA is a corpus of linguistic acceptability for Dutch.",
        reference="https://huggingface.co/datasets/GroNLP/dutch-cola",
        dataset={
            "path": "clips/mteb-nl-dutch-cola",
            "revision": "2269ed7d95d8abaab829f1592b4b2047372e9f81",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2024-03-01", "2024-05-01"),
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="f1",
        domains=["Written"],
        task_subtypes=["Linguistic acceptability"],
        license="not specified",  # specified as unknown
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{gronlp_2024,
  author = {Bylinina, Lisa and Abdi, Silvana and Brouwer, Hylke and Elzinga, Martine and Gunput, Shenza and Huisman, Sem and Krooneman, Collin and Poot, David and Top, Jelmer and Weideman, Cain},
  doi = { 10.57967/hf/3825 },
  publisher = { Hugging Face },
  title = { {Dutch-CoLA (Revision 5a4196c)} },
  url = { https://huggingface.co/datasets/GroNLP/dutch-cola },
  year = {2024},
}
""",
        prompt="Classificeer de gegeven zin als grammaticaal aanvaardbaar of niet aanvaardbaar",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VocalSoundAudioReranking(AbsTaskRetrieval):
    """VocalSound dataset adapted for audio reranking task.

    The VocalSound dataset consists of recordings of human vocal sounds across 6 categories:
    laughter, sighs, coughs, throat clearing, sneezes, and sniffs. The recordings include rich
    metadata such as speaker age, gender, native language, country, and health condition. This
    version is adapted for audio reranking where given a query vocal sound from one of the 6 categories,
    the task is to rank positive audio samples (same vocal sound type) higher than negative
    samples (different vocal sounds).

    Each query has 4 positive examples (same vocal sound category) and 16 negative examples
    (different vocal sound categories), creating a challenging vocal sound retrieval scenario
    with 20 total candidates per query. The dataset contains 198 queries providing robust
    evaluation across all 6 vocal sound categories with excellent speaker diversity.
    """

    metadata = TaskMetadata(
        name="VocalSoundAudioReranking",
        description="VocalSound dataset adapted for audio reranking. Given a query vocal sound from one of 6 categories, rank 4 relevant vocal samples higher than 16 irrelevant ones from different vocal sound types. Contains 198 queries across 6 vocal sound categories for robust evaluation.",
        reference="https://www.researchgate.net/publication/360793875_Vocalsound_A_Dataset_for_Improving_Human_Vocal_Sounds_Recognition/citations",
        dataset={
            "path": "mteb/VocalSoundAudioReranking",
            "revision": "d44884963d55578a13d6da80dfaca0f7f0971fe7",
        },
        type="AudioReranking",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="map_at_1000",
        date=("2022-01-01", "2022-12-31"),
        domains=["Spoken"],
        task_subtypes=["Emotion Reranking"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Gong_2022,
  author = {Gong, Yuan and Yu, Jin and Glass, James},
  booktitle = {ICASSP 2022 - 2022 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
  doi = {10.1109/icassp43922.2022.9746828},
  month = may,
  publisher = {IEEE},
  title = {Vocalsound: A Dataset for Improving Human Vocal Sounds Recognition},
  url = {http://dx.doi.org/10.1109/ICASSP43922.2022.9746828},
  year = {2022},
}
""",
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FGVCAircraftClassification(AbsTaskClassification):
    input_column_name: str = "image"
    samples_per_label: int = 16
    n_experiments: int = 5
    # could be family, manufacturer, or variant. Variant has the higher number of classes.
    label_column_name: str = "variant"

    metadata = TaskMetadata(
        name="FGVCAircraft",
        description="Classifying aircraft images from 41 manufacturers and 102 variants.",
        reference="https://arxiv.org/abs/1306.5151",
        dataset={
            "path": "mteb/FGVCAircraft",
            "revision": "3ae0e4e94e37c610c26df758ec3255fa90d80e67",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2009-01-01",
            "2010-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{maji2013finegrainedvisualclassificationaircraft,
  archiveprefix = {arXiv},
  author = {Subhransu Maji and Esa Rahtu and Juho Kannala and Matthew Blaschko and Andrea Vedaldi},
  eprint = {1306.5151},
  primaryclass = {cs.CV},
  title = {Fine-Grained Visual Classification of Aircraft},
  url = {https://arxiv.org/abs/1306.5151},
  year = {2013},
}
""",
    )



from datasets import concatenate_datasets, load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


def _load_data(path: str, splits: str, revision: str | None = None):
    corpus = {}
    queries = {}
    relevant_docs = {}

    dataset = load_dataset(
        path,
        revision=revision,
    )
    dataset_splits = list(dataset)

    def map_function(split_name):
        return lambda x, idx: {
            "id": f"corpus-{split_name}-{idx}",
            "modality": "image",
        }

    split_datasets = {}
    for split in dataset_splits:
        split_datasets[split] = dataset[split].filter(
            lambda example: example["text_corrected"] is not None
        )

    shared_corpus = concatenate_datasets(
        [
            split_datasets[split].map(
                map_function(split),
                with_indices=True,
                remove_columns=[
                    "split",
                    "text_ocr",
                    "text_corrected",
                    "humor",
                    "sarcasm",
                    "offensive",
                    "motivational",
                    "sentiment",
                ],
            )
            for split in dataset_splits
        ]
    )

    for split in splits:
        corpus[split] = shared_corpus
        split_dataset = split_datasets[split]
        queries[split] = split_dataset.map(
            lambda x, idx: {
                "id": f"query-{split}-{idx}",
                "text": x["text_corrected"],
                "modality": "text",
            },
            with_indices=True,
            remove_columns=[
                "split",
                "text_ocr",
                "humor",
                "sarcasm",
                "offensive",
                "motivational",
                "sentiment",
                "text_corrected",
            ],
        )

        relevant_docs[split] = {}
        for index in range(len(split_dataset)):
            query_id = f"query-{split}-{index}"
            doc_id = f"corpus-{split}-{index}"
            if query_id not in relevant_docs[split]:
                relevant_docs[split][query_id] = {}
            relevant_docs[split][query_id][doc_id] = 1
    return corpus, queries, relevant_docs


class MemotionT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MemotionT2IRetrieval",
        description="Retrieve memes based on captions.",
        reference="https://aclanthology.org/2020.semeval-1.99/",
        dataset={
            "path": "mteb/MMSoc_Memotion",
            "revision": "f77e225ae55c1987b0b8cbf6badd1c10296f5f34",
        },
        type="Any2AnyRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2020-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{sharma2020semeval,
  author = {Sharma, Chhavi and Bhageria, Deepesh and Scott, William and Pykl, Srinivas and Das, Amitava and Chakraborty, Tanmoy and Pulabaigari, Viswanath and Gamb{\"a}ck, Bj{\"o}rn},
  booktitle = {Proceedings of the Fourteenth Workshop on Semantic Evaluation},
  pages = {759--773},
  title = {SemEval-2020 Task 8: Memotion Analysis-the Visuo-Lingual Metaphor!},
  year = {2020},
}
""",
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata.dataset["path"],
            splits=self.metadata.eval_splits,
            revision=self.metadata.dataset["revision"],
        )

        self.data_loaded = True



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.zeroshot_classification import (
    AbsTaskZeroShotClassification,
)


class CLEVR(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="CLEVRZeroShot",
        description="CLEVR closest object distance identification task.",
        reference="https://openaccess.thecvf.com/content_cvpr_2017/html/Johnson_CLEVR_A_Diagnostic_CVPR_2017_paper.html",
        dataset={
            "path": "mteb/wds_vtab-clevr_closest_object_distance",
            "revision": "d2777bb7428d8d74d951b57de0bc2ca5408c4fd4",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2016-01-01", "2016-12-20"),
        domains=["Constructed"],
        task_subtypes=["Object recognition"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{Johnson_2017_CVPR,
  author = {Johnson, Justin and Hariharan, Bharath and van der Maaten, Laurens and Fei-Fei, Li and Lawrence Zitnick, C. and Girshick, Ross},
  booktitle = {Proceedings of the IEEE Conference on Computer Vision and Pattern Recognition (CVPR)},
  month = {July},
  title = {CLEVR: A Diagnostic Dataset for Compositional Language and Elementary Visual Reasoning},
  year = {2017},
}
""",
    )

    input_column_name: str = "webp"
    label_column_name: str = "cls"

    def get_candidate_labels(self) -> list[str]:
        labels = [
            "very nearby",
            "nearby",
            "near",
            "",  # missing this class name in the original dataset: https://huggingface.co/datasets/clip-benchmark/wds_vtab-clevr_closest_object_distance/blob/main/classnames.txt
            "distant",
            "very distant",
        ]

        return [f"{c} shapes." for c in labels]


class CLEVRCount(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="CLEVRCountZeroShot",
        description="CLEVR count objects task.",
        reference="https://openaccess.thecvf.com/content_cvpr_2017/html/Johnson_CLEVR_A_Diagnostic_CVPR_2017_paper.html",
        dataset={
            "path": "mteb/wds_vtab-clevr_count_all",
            "revision": "2e31935f1deb7d22306d3bcb02b76c7cd87a10b3",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2016-01-01", "2016-12-20"),
        domains=["Constructed"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{Johnson_2017_CVPR,
  author = {Johnson, Justin and Hariharan, Bharath and van der Maaten, Laurens and Fei-Fei, Li and Lawrence Zitnick, C. and Girshick, Ross},
  booktitle = {Proceedings of the IEEE Conference on Computer Vision and Pattern Recognition (CVPR)},
  month = {July},
  title = {CLEVR: A Diagnostic Dataset for Compositional Language and Elementary Visual Reasoning},
  year = {2017},
}
""",
    )

    input_column_name: str = "webp"
    label_column_name: str = "cls"

    def get_candidate_labels(self) -> list[str]:
        labels = [
            "three",
            "four",
            "five",
            "six",
            "seven",
            "eight",
            "nine",
            "ten",
        ]
        return [f"a picture of {c} objects" for c in labels]



import torch

from mteb.models import ModelMeta
from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ScoringFunction
from mteb.types import PromptType

F2LLM_CITATION = """@article{2025F2LLM,
    title={F2LLM Technical Report: Matching SOTA Embedding Performance with 6 Million Open-Source Data},
    author={Ziyin Zhang and Zihan Liao and Hang Yu and Peng Di and Rui Wang},
    journal={CoRR},
    volume={abs/2510.02294},
    year={2025},
    url={https://doi.org/10.48550/arXiv.2510.02294},
    doi={10.48550/ARXIV.2510.02294},
    eprinttype={arXiv},
    eprint={2510.02294}
}"""

training_datasets = {
    "MSMARCO",
    "ArguAna",
    "SNLI",
    "MNLI",
    "ANLI",
    "PAQ",
    "SQuAD",
    "StackExchange",
    "NQ",
    "HotpotQA",
    "FEVER",
    "ELI5",
    "FiQA2018",
    "BioASQ",
    "NFCorpus",
    "MIRACLRetrieval",
    "MrTidyRetrieval",
    "SciFact",
    "TriviaQA",
    "COLIEE",
    "PubMedQA",
    "S2ORC",
    "AmazonQA",
    "SPECTER",
    "XSum",
    "CNNDM",
    "SentenceCompression",
    "StackExchangeDupQuestions",
    "QQP",
    "StackOverflowDupQuestions",
    "STS12",
    "STS22",
    "STSBenchmark",
    "AmazonCounterfactualClassification",
    "AmazonPolarityClassification",
    "ImdbClassification",
    "ToxicConversationsClassification",
    "CoLA",
    "AmazonReviewClassification",
    "Banking77Classification",
    "EmotionClassification",
    "MTOPIntentClassification",
    "MTOPDomainClassification",
    "MassiveScenarioClassification",
    "MassiveIntentClassification",
    "TweetSentimentExtractionClassification",
    "ArxivClusteringP2P",
    "ArxivClusteringS2S",
    "BiorxivClusteringP2P",
    "BiorxivClusteringS2S",
    "MedrxivClusteringP2P",
    "MedrxivClusteringS2S",
    "RedditClustering",
    "RedditClusteringP2P",
    "StackExchangeClustering",
    "StackExchangeClusteringP2P",
    "TwentyNewsgroupsClustering",
}

f2llmv2_training_datasets = training_datasets.union(
    {
        "Waimai",
        "NordicClassification",
        "SIB200",
        "MLSUMClustering",
        "BactrianXLanguageClassification",
        "BactrianXTranslation",
        "Europarl",
        "XCodeEvalTranslation",
        "TNews",
        "THUCNews",
        "CSL",
        "CEDR",
        "RuSentimentClustering",
        "WebFAQ",
        "MMARCO",
        "MLDR",
        "MKQA",
        "MLSUMRetrieval",
        "Aya",
        "MURI",
        "UNPC",
        "XNLI",
        "PawsX",
        "CodeSearchNet",
        "CodeSearchNetCCR",
        "CosQA",
        "SyntheticText2SQL",
        "CodeFeedbackMT",
        "CodeFeedbackST",
        "StackOverflowQA",
        "OpenCodeGeneticInstruct",
        "OpenCodeReasoning2",
        "XCodeEvalCodeToCode",
        "XCodeEvalNLToCode",
        "ProCQA",
        "ParaCrawl",
        "NaturalReasoning",
        "MultiAlpaca",
        "COIG",
        "OASST2",
        "WildChat",
        "M2Lingual",
        "InfinityInstruct",
        "cMedQAv2",
        "MedInstruct",
        "HuatuoKGQA",
        "HuatuoEncQA",
        "CLIRMatrix",
        "BQ",
        "YahooAnswers",
        "DuReader",
        "T2Ranking",
        "GooAQ",
        "MedicalQARu",
        "MEDI2",
        "OpenOrca",
        "MedicalInstruction",
        "MultiCPRMedical",
        "MultiCPRECom",
        "HealthCareMagic",
        "OCNLI",
        "ESCI",
        "CORD19",
        "CMCQA",
        "CMNLI",
        "DBPedia",
        "FEVERNL",
        "HotpotQANL",
        "KoAlpaca",
        "KoAlpacaRealQA",
        "KoMagpie",
        "Lawzhidao",
        "LCQMC",
        "QBQTC",
        "LCSTS",
        "LLMRetrievalData",
        "MailruQA",
        "MedicalFlashcards",
        "MedMCQA",
        "MedQA",
        "MedQuAD",
        "MSciNLI",
        "NordicRetrieval",
        "NordicSTS",
        "NordicTextMatching",
        "ParSQuAD",
        "PersianQA",
        "PQuAD",
        "RefGPT",
        "RuInstruct",
        "SiberianDataset",
        "SimCLUE",
        "webMedQA",
        "WikiOmnia",
        "MQA",
    }
)

c2llm_training_datasets = {
    "CodeSearchNet",
    "CodeSearchNetRetrieval",
    "CodeSearchNetCCRetrieval",
    "CodeEditSearchRetrieval",
    "CodeFeedbackMT",
    "CodeFeedbackST",
    "CodeTransOceanContest",
    "CodeTransOceanDL",
    "COIRCodeSearchNetRetrieval",
    "CosQA",
    "StackOverflowQA",
    "SyntheticText2SQL",
    "AdvTrain",
}

prompts_dict = {
    "AmazonCounterfactualClassification": "Classify a given Amazon customer review text as either counterfactual or not counterfactual.",
    "Banking77Classification": "Given an online banking query, find the corresponding intents.",
    "ImdbClassification": "Classify the sentiment expressed in the given movie review text from the IMDB dataset.",
    "MTOPDomainClassification": "Classify the intent domain of the given utterance in task-oriented conversation.",
    "MassiveIntentClassification": "Given a user utterance as query, find the user intents.",
    "MassiveScenarioClassification": "Given a user utterance as query, find the user scenarios.",
    "ToxicConversationsClassification": "Classify the given comments as either toxic or not toxic.",
    "TweetSentimentExtractionClassification": "Classify the sentiment of a given tweet as either positive, negative, or neutral",
    "ArXivHierarchicalClusteringP2P": "Identify the main and secondary category of arXiv papers based on the titles and abstracts.",
    "ArXivHierarchicalClusteringS2S": "Identify the main and secondary category of arXiv papers based on the titles.",
    "BiorxivClusteringP2P.v2": "Identify the main category of bioRxiv papers based on the titles and abstracts.",
    "MedrxivClusteringP2P.v2": "Identify the main category of medRxiv papers based on the titles and abstracts.",
    "MedrxivClusteringS2S.v2": "Identify the main category of medRxiv papers based on the titles.",
    "StackExchangeClustering.v2": "Identify the topic or theme of StackExchange posts based on the titles.",
    "StackExchangeClusteringP2P.v2": "Identify the topic or theme of StackExchange posts based on the given paragraphs.",
    "TwentyNewsgroupsClustering.v2": "Identify the topic or theme of the given news articles.",
    "SprintDuplicateQuestions": "Retrieve duplicate questions from Sprint forum.",
    "TwitterSemEval2015": "Retrieve tweets that are semantically similar to the given tweet.",
    "TwitterURLCorpus": "Retrieve tweets that are semantically similar to the given tweet.",
    "AskUbuntuDupQuestions": "Retrieve duplicate questions from AskUbuntu forum.",
    "MindSmallReranking": "Retrieve relevant news articles based on user browsing history.",
    "ArguAna": "Given a claim, find documents that refute the claim.",
    "CQADupstackGamingRetrieval": "Given a question, retrieve questions that are semantically equivalent.",
    "CQADupstackUnixRetrieval": "Given a question, retrieve questions that are semantically equivalent.",
    "ClimateFEVERHardNegatives": "Given a claim about climate change, retrieve documents that support or refute the claim.",
    "FEVERHardNegatives": "Given a claim, retrieve documents that support or refute the claim.",
    "FiQA2018": "Given a financial question, retrieve passages that answer the question.",
    "HotpotQAHardNegatives": "Given a multi-hop question, retrieve passages that answer the question.",
    "SCIDOCS": "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper.",
    "TRECCOVID": "Given a query on COVID-19, retrieve documents that answer the query.",
    "Touche2020Retrieval.v3": "Given a question, retrieve passages that answer the question.",
    "BIOSSES": "Retrieve semantically similar text.",
    "SICK-R": "Retrieve semantically similar text.",
    "STS12": "Retrieve semantically similar text.",
    "STS13": "Retrieve semantically similar text.",
    "STS14": "Retrieve semantically similar text.",
    "STS15": "Retrieve semantically similar text.",
    "STS17": "Retrieve semantically similar text.",
    "STS22.v2": "Retrieve semantically similar text.",
    "STSBenchmark": "Retrieve semantically similar text.",
    "SummEvalSummarization.v2": "Given a news summary, retrieve other semantically similar summaries.",
}

f2llmv2_prompts_dict = prompts_dict.copy()
f2llmv2_prompts_dict.update(
    {
        "STS": "Retrieve semantically similar text.",
        "BitextMining": "Retrieve translations of the following text.",
        "Classification": "Classify the topic of the given text.",
        "MultilabelClassification": "Classify the topic of the given text.",
        "Clustering": "Classify the topic of the given text.",
        "PairClassification": "Retrieve semantically entailed text.",
        "query": "Given a question, retrieve passages that can help answer the question.",
        "document": "",
        "BulgarianStoreReviewSentimentClassfication": "Classify the sentiment of the given text.",
        "CzechProductReviewSentimentClassification": "Classify the sentiment of the given text.",
        "FinancialPhrasebankClassification": "Classify the sentiment of the given text.",
        "PoemSentimentClassification": "Classify the sentiment of the given text.",
        "EstonianValenceClassification": "Classify the sentiment of the given text.",
        "FilipinoShopeeReviewsClassification": "Classify the sentiment of the given text.",
        "SentimentAnalysisHindi": "Classify the sentiment of the given text.",
        "KurdishSentimentClassification": "Classify the sentiment of the given text.",
        "MacedonianTweetSentimentClassification": "Classify the sentiment of the given text.",
        "AfriSentiClassification": "Classify the sentiment of the given text.",
        "CataloniaTweetClassification": "Classify the sentiment of the given text.",
        "MultiHateClassification": "Classify the sentiment of the given text.",
        "NusaParagraphEmotionClassification": "Classify the sentiment of the given text.",
        "NusaX-senti": "Classify the sentiment of the given text.",
        "SwissJudgementClassification": "Classify the sentiment of the given text.",
        "PolEmo2.0-OUT": "Classify the sentiment of the given text.",
        "CSFDSKMovieReviewSentimentClassification": "Classify the sentiment of the given text.",
        "SlovakMovieReviewSentimentClassification": "Classify the sentiment of the given text.",
        "CyrillicTurkicLangClassification": "Classify the text into its language.",
        "IndicLangClassification": "Classify the text into its language.",
        "NordicLangClassification": "Classify the text into its language.",
        "ScalaClassification": "Classify the given sentence as linguistically acceptable or not acceptable.",
        "DalajClassification": "Classify the given sentence as linguistically acceptable or not acceptable.",
        "IndonesianIdClickbaitClassification": "Classify the given text as either clickbait or not clickbait.",
        "KorSarcasmClassification": "Classify the given text as either sarcasm or not sarcasm.",
        "PAC": "Classify the given clause as either abusive or not abusive.",
        "WikiCitiesClustering": "Classify the following city description by country.",
        "ArmenianParaphrasePC": "Retrieve paraphrases of the given sentence.",
        "OpusparcusPC": "Retrieve paraphrases of the given sentence.",
        "PawsXPairClassification": "Retrieve paraphrases of the given sentence.",
        "PpcPC": "Retrieve paraphrases of the given sentence.",
        "KorHateSpeechMLClassification": "Classify the sentiment of the given text.",
        "CEDRClassification": "Classify the emotion expressed in the given comment into: joy, sadness, surprise, fear, and anger.",
        "AppsRetrieval": "Retrieve the most relevant code snippet for the given query.",
        "CodeEditSearchRetrieval": "Retrieve the most relevant code edit.",
        "CodeFeedbackMT": "Retrieve the most relevant response for the given query.",
        "CodeFeedbackST": "Retrieve the most relevant response for the given query.",
        "CodeSearchNetCCRetrieval": "Retrieve the most relevant code snippet for the given code snippet.",
        "CodeSearchNetRetrieval": "Retrieve the most relevant code snippet for the given query.",
        "CodeTransOceanContest": "Retrieve similar code to the given source code.",
        "CodeTransOceanDL": "Retrieve similar code to the given source code.",
        "CosQA": "Retrieve the most relevant code snippet for the given query.",
        "COIRCodeSearchNetRetrieval": "Retrieve the most relevant code summary for the given code snippet.",
        "StackOverflowQA": "Retrieve the most relevant response for the given query.",
        "SyntheticText2SQL": "Retrieve the most relevant sql code snippet for the given query.",
        "AILAStatutes": "Identify the most relevant statutes for the given situation.",
        "LegalBenchCorporateLobbying": "Given a bill title, retrieve the corresponding bill summary.",
        "CovidRetrieval": "Given a query on COVID-19, retrieve documents that answer the query.",
        "WebLINXCandidatesReranking": "Given a web navigation step, retrieve relevant elements.",
        "NFCorpus": "Given a question, retrieve passages that answer the question.",
        "TRECCOVID-PL": "Given a query on COVID-19, retrieve documents that answer the query.",
        "SciFact": "Given a scientific claim, retrieve passages that support or refute the claim.",
        "SciFact-PL": "Given a scientific claim, retrieve passages that support or refute the claim.",
        "CmedqaRetrieval": "Given a question, retrieve passages that answer the question.",
        "CMedQAv2-reranking": "Given a question, retrieve passages that answer the question.",
        "AngryTweetsClassification": "Classify the sentiment of the given text.",
        "DanishPoliticalCommentsClassification": "Classify the sentiment of the given text.",
        "DKHateClassification": "Classify the given comments as either offensive or not offensive.",
        "LccSentimentClassification": "Classify the sentiment of the given text.",
        "NoRecClassification": "Classify the sentiment of the given text.",
        "NorwegianParliamentClassification": "Classify the sentiment of the given text.",
        "SwedishSentimentClassification": "Classify the sentiment of the given text.",
        "SweRecClassification": "Classify the sentiment of the given text.",
        "DanFeverRetrieval": "Given a claim, retrieve documents that support or refute the claim.",
        "SNLRetrieval": "Given a summary, retrieve the original article.",
        "SwednRetrieval": "Given a summary, retrieve the original article.",
        "TV2Nordretrieval": "Given a news summary, retrieve the original article.",
        "BengaliSentimentAnalysis": "Classify the sentiment of the given text.",
        "HindiDiscourseClassification": "Classify the given text into one of the five discourse modes: argumentative, narrative, descriptive, dialogic, and informative.",
        "MTOPIntentClassification": "Classify the intent of the given utterance in task-oriented conversation.",
        "TweetSentimentClassification": "Classify the sentiment of the given text.",
        "UrduRomanSentimentClassification": "Classify the sentiment of the given text as either positive, negative, or neutral.",
        "AmazonReviewsClassification": "Classify the given Amazon review into its appropriate rating category.",
        "BlurbsClusteringP2P": "Classify the given book title and blurb into its genre.",
        "BlurbsClusteringS2S": "Classify the given book title into its genre.",
        "FalseFriendsGermanEnglish": "Retrieve translations of the following text.",
        "XMarket": "Given a product name search, retrieve the corresponding product description.",
        "GerDaLIR": "Retrieve documents that are referenced by the given text.",
        "MLSUMClusteringP2P": "Classify the topic of the given news article.",
        "SummEvalFr": "Given a news summary, retrieve other semantically similar summaries.",
        "AllegroReviews": "Classify the sentiment of the given text.",
        "CBD": "Classify the given text as either cyberbullying or not.",
        "PolEmo2.0-IN": "Classify the sentiment of the given text.",
        "PSC": "Retrieve semantically similar text.",
        "EcomRetrieval": "Given a product name query, retrieve the corresponding product description.",
        "MedicalRetrieval": "Retrieve the most relevant response for the given query.",
        "VideoRetrieval": "Given a video search query, retrieve the titles of relevant videos.",
        "CMedQAv1-reranking": "Retrieve the most relevant response for the given query.",
        "Waimai": "Classify the sentiment of the given review as either positive or negative.",
        "OnlineShopping": "Classify the sentiment of the given review as either positive or negative.",
        "JDReview": "Classify the sentiment of the given review as either positive or negative.",
        "MultilingualSentiment": "Classify the sentiment of the given review as either positive, negative, or neutral.",
        "ToxicChatClassification": "Classify the given text as either toxic or not toxic.",
        "JapaneseSentimentClassification": "Classify the sentiment of the given text.",
        "WRIMEClassification": "Classify the sentiment of the given text.",
        "NLPJournalTitleAbsRetrieval.V2": "Given a paper's title, retrieve the corresponding abstract.",
        "NLPJournalTitleIntroRetrieval.V2": "Given a paper's title, retrieve the corresponding introduction.",
        "NLPJournalAbsIntroRetrieval.V2": "Given a paper's abstract, retrieve the corresponding introduction.",
        "NLPJournalAbsArticleRetrieval.V2": "Given a paper's abstract, retrieve the corresponding paper.",
        "ESCIReranking": "Given a product name query, retrieve the corresponding product description.",
        "DutchBookReviewSentimentClassification.v2": "Classify the sentiment of the given text.",
        "VaccinChatNLClassification": "Classify the intent of the given utterance.",
        "DutchColaClassification": "Classify the given sentence as linguistically acceptable or not acceptable.",
        "DutchGovernmentBiasClassification": "Classify the given government document as biased or unbiased.",
        "DutchSarcasticHeadlinesClassification": "Classify the given newspaper headline as sarcastic or not sarcastic.",
        "XLWICNLPairClassification": "Retrieve semantically similar text.",
        "CovidDisinformationNLMultiLabelClassification": "Classify the given social media post related to COVID-19 into its misinformation category.",
        "VABBClusteringS2S": "Identify the main category of the given paper based on the title.",
        "VABBClusteringP2P": "Identify the main category of the given paper based on the title and abstract.",
        "ArguAna-NL.v2": "Given a claim, find documents that refute the claim.",
        "SCIDOCS-NL.v2": "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper.",
        "SciFact-NL.v2": "Given a scientific claim, retrieve passages that support or refute the claim.",
        "DutchNewsArticlesRetrieval": "Given a news title, retrieve the original article.",
        "OpenTenderRetrieval": "Given a title, retrieve the corresponding article.",
        "VABBRetrieval": "Given a paper's title, retrieve the corresponding abstract.",
        "GeoreviewClassification": "Classify the given review into its appropriate rating category.",
        "InappropriatenessClassification": "Classify the given message as either sensitive topic or not.",
        "KinopoiskClassification": "Classify the sentiment of the given movie review.",
        "RuReviewsClassification": "Classify the sentiment of the given review as either positive, negative, or neutral.",
        "RuSciBenchGRNTIClassification": "Identify the main category of the given paper based on the title and abstract.",
        "RuSciBenchOECDClassification": "Identify the main category of the given paper based on the title and abstract.",
        "GeoreviewClusteringP2P": "Identify the organization category based on the given review.",
        "RuSciBenchGRNTIClusteringP2P": "Identify the main category of the given paper based on the title and abstract.",
        "RuSciBenchOECDClusteringP2P": "Identify the main category of the given paper based on the title and abstract.",
        "SensitiveTopicsClassification": "Classify the given text into sensitive topics.",
        "RiaNewsRetrievalHardNegatives.v2": "Given a news title, retrieve the original article.",
        "PersianFoodSentimentClassification": "Classify the sentiment of the given text as either positive or negative.",
        "SynPerChatbotConvSAClassification": "Classify the sentiment of the given text.",
        "SynPerChatbotConvSAToneChatbotClassification": "Classify the sentiment of the given text.",
        "SynPerChatbotConvSAToneUserClassification": "Classify the sentiment of the given text.",
        "SynPerChatbotSatisfactionLevelClassification": "Classify the satisfaction level of the given text.",
        "SynPerTextToneClassification.v3": "Classify the tone of the given text.",
        "DeepSentiPers.v2": "Classify the sentiment of the given text.",
        "PersianTextEmotion.v2": "Classify the emotion expressed in the given text into: joy, sadness, surprise, fear, anger, and love.",
        "StyleClassification": "Classify the style of the given text as either formal or informal.",
        "PerShopDomainClassification": "Classify the domain of the given utterance in shopping dialogue.",
        "PerShopIntentClassification": "Classify the intent of the given utterance in shopping dialogue.",
        "SynPerChatbotRAGFAQPC": "Retrieve the most relevant response for the given query.",
        "FarsiParaphraseDetection": "Retrieve semantically similar text.",
        "SynPerTextKeywordsPC": "Identify keywords in the given text.",
        "SynPerQAPC": "Retrieve the most relevant response for the given query.",
        "ParsinluQueryParaphPC": "Retrieve semantically similar text.",
        "SynPerChatbotRAGFAQRetrieval": "Retrieve the most relevant response for the given query.",
        "HotpotQA-FaHardNegatives": "Given a multi-hop question, retrieve passages that answer the question.",
        "ArguAna-Fa.v2": "Given a claim, find documents that refute the claim.",
        "QuoraRetrieval-Fa.v2": "Retrieve questions that are semantically equivalent to the given one.",
        "SCIDOCS-Fa.v2": "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper.",
        "SciFact-Fa.v2": "Given a scientific claim, retrieve passages that support or refute the claim.",
        "TRECCOVID-Fa.v2": "Given a query on COVID-19, retrieve documents that answer the query.",
        "FEVER-FaHardNegatives": "Given a claim, retrieve documents that support or refute the claim.",
        "SAMSumFa": "Retrieve the most relevant summary for the given conversation.",
        "SynPerChatbotSumSRetrieval": "Retrieve the most relevant summary for the given conversation.",
        "SynPerChatbotRAGSumSRetrieval": "Retrieve the most relevant summary for the given conversation.",
        "ArguAna-VN": "Given a claim, find documents that refute the claim.",
        "SciFact-VN": "Given a scientific claim, retrieve passages that support or refute the claim.",
        "ClimateFEVER-VN": "Given a claim about climate change, retrieve documents that support or refute the claim.",
        "FEVER-VN": "Given a claim, retrieve documents that support or refute the claim.",
        "DBPedia-VN": "Given a query, retrieve relevant entity descriptions.",
        "HotpotQA-VN": "Given a multi-hop question, retrieve passages that answer the question.",
        "TRECCOVID-VN": "Given a query on COVID-19, retrieve documents that answer the query.",
        "SCIDOCS-VN": "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper.",
        "Quora-VN": "Retrieve questions that are semantically equivalent to the given one.",
        "CQADupstackAndroid-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackGis-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackMathematica-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackPhysics-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackProgrammers-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackStats-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackTex-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackUnix-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackWebmasters-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "CQADupstackWordpress-VN": "Given a question, retrieve questions that are semantically equivalent.",
        "Banking77VNClassification": "Given an online banking query, find the corresponding intents.",
        "EmotionVNClassification": "Classify the emotion expressed in the given Twitter message into one of the six emotions: anger, fear, joy, love, sadness, and surprise.",
        "AmazonCounterfactualVNClassification": "Classify a given Amazon customer review text as either counterfactual or not counterfactual.",
        "MTOPDomainVNClassification": "Classify the intent domain of the given utterance in task-oriented conversation.",
        "TweetSentimentExtractionVNClassification": "Classify the sentiment of a given tweet as either positive, negative, or neutral",
        "ToxicConversationsVNClassification": "Classify the given comments as either toxic or not toxic.",
        "ImdbVNClassification": "Classify the sentiment expressed in the given movie review text from the IMDB dataset.",
        "MTOPIntentVNClassification": "Classify the intent of the given utterance in task-oriented conversation.",
        "MassiveScenarioVNClassification": "Given a user utterance as query, find the user scenarios.",
        "MassiveIntentVNClassification": "Given a user utterance as query, find the user intents.",
        "AmazonReviewsVNClassification": "Classify the given Amazon review into its appropriate rating category.",
        "AmazonPolarityVNClassification": "Classify the given Amazon review as either positive or negative.",
        "SprintDuplicateQuestions-VN": "Retrieve duplicate questions from Sprint forum.",
        "TwitterSemEval2015-VN": "Retrieve tweets that are semantically similar to the given tweet.",
        "TwitterURLCorpus-VN": "Retrieve tweets that are semantically similar to the given tweet.",
        "TwentyNewsgroupsClustering-VN": "Identify the topic or theme of the given news articles.",
        "RedditClusteringP2P-VN": "Identify the topic or theme of Reddit posts based on the titles and posts.",
        "StackExchangeClusteringP2P-VN": "Identify the topic or theme of StackExchange posts based on the given titles and paragraphs.",
        "StackExchangeClustering-VN": "Identify the topic or theme of StackExchange posts based on the titles.",
        "RedditClustering-VN": "Identify the topic or theme of Reddit posts based on the titles.",
        "SciDocsRR-VN": "Given a title of a scientific paper, retrieve the titles of other relevant papers.",
        "AskUbuntuDupQuestions-VN": "Retrieve duplicate questions from AskUbuntu forum.",
        "StackOverflowDupQuestions-VN": "Retrieve duplicate questions from StackOverflow forum.",
    }
)


c2llm_prompts_dict = {
    "CodeEditSearchRetrieval": {
        "query": "Retrieve the diff code that relevant the following query:\n",
        "document": "Retrieved Answer:",
    },
    "CodeSearchNetRetrieval": {
        "query": "Retrieve the code that solves the following query:\n",
        "document": "Retrieved Answer:",
    },
    "AppsRetrieval": {
        "query": "Given a problem description from a programming contest, retrieve code examples that can assist in solving it.\n",
        "document": "Retrieved Answer:",
    },
    "CodeFeedbackMT": {
        "query": "Given a multi-turn conversation history that includes both text and code, retrieve relevant multi-modal answers composed of text and code that address the ongoing discussion.\n",
        "document": "Retrieved Answer:",
    },
    "CodeFeedbackST": {
        "query": "Given a single-turn question composed of text and code, retrieve suitable answers that also mix text and code to provide helpful feedback.\n",
        "document": "Retrieved Answer:",
    },
    "CodeSearchNetCCRetrieval": {
        "query": "Given an initial code segment, retrieve the subsequent segment that continues the code.\n",
        "document": "Retrieved Answer:",
    },
    "CodeTransOceanContest": {
        "query": "Given a Python code snippet, retrieve its semantically equivalent version written in C++.\n",
        "document": "Retrieved Answer:",
    },
    "CodeTransOceanDL": {
        "query": "Given a Python code snippet, retrieve its semantically equivalent version written in C++.\n",
        "document": "Retrieved Answer:",
    },
    "COIRCodeSearchNetRetrieval": {
        "query": "Given a code snippet, retrieve its corresponding document string that summarizes its functionality.\n",
        "document": "Retrieved Answer:",
    },
    "CosQA": {
        "query": "Given a query from a web search, retrieve code that is helpful in addressing the query.\n",
        "document": "Retrieved Answer:",
    },
    "StackOverflowQA": {
        "query": "Given a question combining text and code, retrieve relevant answers that also contain both text and code snippets and can address the question.\n",
        "document": "Retrieved Answer:",
    },
    "SyntheticText2SQL": {
        "query": "Given a natural language question, retrieve SQL queries that serve as appropriate responses.\n",
        "document": "Retrieved Answer:",
    },
}

c2llm_languages = [
    "eng-Latn",
    "zho-Hans",
    "python-Code",
    "javascript-Code",
    "go-Code",
    "ruby-Code",
    "java-Code",
    "php-Code",
]

f2llmv2_languages = [
    "python-Code",
    "php-Code",
    "java-Code",
    "c++-Code",
    "go-Code",
    "javascript-Code",
    "c#-Code",
    "ruby-Code",
    "c-Code",
    "rust-Code",
    "sql-Code",
    "eng-Latn",
    "zho-Hans",
    "rus-Cyrl",
    "spa-Latn",
    "fra-Latn",
    "deu-Latn",
    "ara-Arab",
    "nld-Latn",
    "vie-Latn",
    "hin-Deva",
    "kor-Hang",
    "jpn-Jpan",
    "ita-Latn",
    "ind-Latn",
    "por-Latn",
    "pol-Latn",
    "tur-Latn",
    "dan-Latn",
    "tha-Thai",
    "swe-Latn",
    "fas-Arab",
    "ukr-Cyrl",
    "ces-Latn",
    "nor-Latn",
    "ell-Grek",
    "cat-Latn",
    "ron-Latn",
    "fin-Latn",
    "bul-Cyrl",
    "tgl-Latn",
    "glg-Latn",
    "mya-Mymr",
    "hye-Armn",
    "khm-Khmr",
    "nep-Deva",
    "hun-Latn",
    "eus-Latn",
    "heb-Hebr",
    "lao-Laoo",
    "swa-Latn",
    "azj-Latn",
    "lav-Latn",
    "sin-Sinh",
    "slk-Latn",
    "tgk-Cyrl",
    "est-Latn",
    "lit-Latn",
    "msa-Latn",
    "hrv-Latn",
    "isl-Latn",
    "slv-Latn",
    "srp-Cyrl",
    "urd-Arab",
    "ben-Beng",
    "aze-Latn",
    "afr-Latn",
    "tam-Taml",
    "kat-Geor",
    "tel-Telu",
    "mal-Mlym",
    "mon-Cyrl",
    "nno-Latn",
    "kaz-Cyrl",
    "cym-Latn",
    "mar-Deva",
    "sqi-Latn",
    "nob-Latn",
    "pus-Arab",
    "mkd-Cyrl",
    "ceb-Latn",
    "jav-Latn",
    "war-Latn",
    "kan-Knda",
    "epo-Latn",
    "lat-Latn",
    "guj-Gujr",
    "uzb-Latn",
    "amh-Ethi",
    "oci-Latn",
    "bel-Cyrl",
    "azb-Arab",
]


c2llm_loader_kwargs = dict(
    trust_remote_code=True,
    prompts_dict=c2llm_prompts_dict,
    apply_instruction_to_passages=True,
    max_seq_length=2048,
    padding_side="left",
)


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    if not instruction or prompt_type == PromptType.document:
        return ""
    if isinstance(instruction, dict):
        if prompt_type is None:
            instruction = "Given a web search query, retrieve relevant passages that answer the query"
        else:
            instruction = instruction[prompt_type]
    return f"Instruct: {instruction}\nQuery: "


F2LLM_0B6 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        prompts_dict=prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
    ),
    name="codefuse-ai/F2LLM-0.6B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="36416618b83d4bd84a8ca30c2ee01ed518f9f2e7",
    release_date="2025-09-18",
    n_parameters=596049920,
    n_embedding_parameters=155582464,
    memory_usage_mb=1137,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=8192,
    reference="https://huggingface.co/codefuse-ai/F2LLM-0.6B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM",
    training_datasets=training_datasets,
    citation=F2LLM_CITATION,
)

F2LLM_1B7 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        prompts_dict=prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
    ),
    name="codefuse-ai/F2LLM-1.7B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="fdce0e09655f42cea26f7f66f5a70cd4507ea45c",
    release_date="2025-09-18",
    n_parameters=1_720_574_976,
    n_embedding_parameters=311164928,
    memory_usage_mb=3282,
    embed_dim=2048,
    license="apache-2.0",
    max_tokens=8192,
    reference="https://huggingface.co/codefuse-ai/F2LLM-1.7B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM",
    training_datasets=training_datasets,
    citation=F2LLM_CITATION,
)

F2LLM_4B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        prompts_dict=prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
    ),
    name="codefuse-ai/F2LLM-4B",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="9fe95901ed2b6b59dd7673d6e93c9d76766a1e25",
    release_date="2025-09-18",
    n_parameters=4022468096,
    n_embedding_parameters=388956160,
    memory_usage_mb=7672,
    embed_dim=2560,
    license="apache-2.0",
    max_tokens=8192,
    reference="https://huggingface.co/codefuse-ai/F2LLM-4B",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM",
    training_datasets=training_datasets,
    citation=F2LLM_CITATION,
)

C2LLM_0B5 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=c2llm_loader_kwargs,
    name="codefuse-ai/C2LLM-0.5B",
    revision="f08c18be03de42c6e388948a1804d4b271a953a2",
    release_date="2025-12-22",
    languages=c2llm_languages,
    n_parameters=497252096,
    n_embedding_parameters=136134656,
    memory_usage_mb=948.0,
    max_tokens=32768,
    embed_dim=896,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers", "safetensors"],
    reference="https://huggingface.co/codefuse-ai/C2LLM-0.5B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=c2llm_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    is_cross_encoder=None,
    citation=None,
    contacts=None,
)

C2LLM_7B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=c2llm_loader_kwargs,
    name="codefuse-ai/C2LLM-7B",
    revision="c1dc16d6d64eb962c783bfb36a6d9c2f24a86dca",
    release_date="2025-12-22",
    languages=c2llm_languages,
    n_parameters=7667028992,
    n_embedding_parameters=544997376,
    memory_usage_mb=14624.0,
    max_tokens=32768,
    embed_dim=3584,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers", "Transformers", "safetensors"],
    reference="https://huggingface.co/codefuse-ai/C2LLM-7B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=c2llm_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    is_cross_encoder=None,
    citation=None,
    contacts=None,
)

F2LLM_v2_80M = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-80M",
    revision="35baae5091e26211dbe628f1723448c146fafdf2",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=80084288,
    n_active_parameters_override=None,
    n_embedding_parameters=48534080,
    memory_usage_mb=305,
    max_tokens=40960,
    embed_dim=320,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-80M",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_160M = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-160M",
    revision="9eb35007d12e8624e6bf3e9e35d2255b5269597f",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=159185024,
    n_active_parameters_override=None,
    n_embedding_parameters=97068160,
    memory_usage_mb=607,
    max_tokens=40960,
    embed_dim=640,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-160M",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_330M = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-330M",
    revision="b0e8e19bdf603b9826422852572c0d18d482c414",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=334349184,
    n_active_parameters_override=None,
    n_embedding_parameters=135895424,
    memory_usage_mb=1275,
    max_tokens=40960,
    embed_dim=896,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-330M",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_0B6 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-0.6B",
    revision="54b4e2dc74e01be7126d4cf5f016af6b21edc563",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=596049920,
    n_active_parameters_override=None,
    n_embedding_parameters=155309056,
    memory_usage_mb=2274,
    max_tokens=40960,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-0.6B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_1B7 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-1.7B",
    revision="3766d46e7a68545ed6190c15330983f9b39ab718",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=1720574976,
    n_active_parameters_override=None,
    n_embedding_parameters=310618112,
    memory_usage_mb=6563,
    max_tokens=40960,
    embed_dim=2048,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-1.7B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_4B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-4B",
    revision="e04d1a04f4a0e154bf43969d3dcc596bbd92b1f8",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=4022468096,
    n_active_parameters_override=None,
    n_embedding_parameters=388272640,
    memory_usage_mb=15344,
    max_tokens=40960,
    embed_dim=2560,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-4B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_8B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-8B",
    revision="5ac7d57351026337281e8e6a23e05cce16e79415",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=7568405504,
    n_active_parameters_override=None,
    n_embedding_parameters=621236224,
    memory_usage_mb=28871,
    max_tokens=40960,
    embed_dim=4096,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-8B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)

F2LLM_v2_14B = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template="Instruct: {instruction}\nQuery: ",
        prompts_dict=f2llmv2_prompts_dict,
        apply_instruction_to_passages=False,
        add_eos_token=True,
        max_seq_length=8192,
        model_kwargs={
            "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.bfloat16,
        },
    ),
    name="codefuse-ai/F2LLM-v2-14B",
    revision="e42037a37e12999db44ce8e1a0e63c366c273f6a",
    release_date="2026-03-09",
    languages=f2llmv2_languages,
    n_parameters=13990394880,
    n_active_parameters_override=None,
    n_embedding_parameters=776545280,
    memory_usage_mb=53369,
    max_tokens=40960,
    embed_dim=5120,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/codefuse-ai/F2LLM",
    public_training_data="https://huggingface.co/datasets/codefuse-ai/F2LLM-v2",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    reference="https://huggingface.co/codefuse-ai/F2LLM-v2-14B",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=f2llmv2_training_datasets,
    adapted_from=None,
    superseded_by=None,
    modalities=["text"],
    model_type=["dense"],
    citation=None,
    contacts=None,
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FSDD(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FSDD",
        description="Spoken digit classification of audio into one of 10 classes: 0-9",
        reference="https://huggingface.co/datasets/silky1708/Free-Spoken-Digit-Dataset",
        dataset={
            "path": "mteb/free-spoken-digit-dataset",
            "revision": "c34455c99604d35cb8d27328c267be1478efc903",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2020-01-01", "2020-10-06"),
        domains=["Music"],
        task_subtypes=["Spoken Digit Classification"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{zohar2018free,
  author = {J. Zohar and S. Cãar and F. Jason and P. Yuxin and N. Hereman and T. Adhish},
  month = {aug},
  title = {Jakobovski/Free-Spoken-Digit-Dataset: V1.0.8},
  url = {https://doi.org/10.5281/zenodo.1342401},
  year = {2018},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"



from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import (
    SentenceTransformerEncoderWrapper,
)

dfm_enc_large = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="KennethEnevoldsen/dfm-sentence-encoder-large",
    model_type=["dense"],
    languages=["dan-Latn"],
    open_weights=True,
    revision="132c53391e7a780dc6a2f9a03724d0158fe7122c",
    release_date="2023-07-12",
    n_parameters=355087360,
    n_embedding_parameters=51_200_000,
    memory_usage_mb=1554,
    embed_dim=1024,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/KennethEnevoldsen/dfm-sentence-encoder-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="chcaa/dfm-encoder-large-v1",
    training_datasets=set(),  # just contrastive pre-training
    public_training_code="https://huggingface.co/KennethEnevoldsen/dfm-sentence-encoder-large#hyperparameters",
    citation="""@article{enevoldsenScandinavianEmbeddingBenchmarks2024,
    title = {The {Scandinavian} {Embedding} {Benchmarks}: {Comprehensive} {Assessment} of {Multilingual} and {Monolingual} {Text} {Embedding}},
    shorttitle = {The {Scandinavian} {Embedding} {Benchmarks}},
    url = {https://openreview.net/forum?id=pJl_i7HIA72},
    language = {en},
    urldate = {2024-04-12},
    author = {Enevoldsen, Kenneth and Kardos, Márton and Muennighoff, Niklas and Nielbo, Kristoffer},
    month = feb,
    year = {2024},
}
""",
    public_training_data="https://huggingface.co/datasets/danish-foundation-models/danish-gigaword",  # paragraphs extracted from Danish Gigaword
)

dfm_enc_med = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="KennethEnevoldsen/dfm-sentence-encoder-medium",
    model_type=["dense"],
    languages=["dan-Latn"],
    open_weights=True,
    revision="701bce95d499fa97610d57e8823c54fd1fb79930",
    release_date="2023-07-12",
    n_parameters=124445952,
    n_embedding_parameters=38_403_840,
    memory_usage_mb=475,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/KennethEnevoldsen/dfm-sentence-encoder-medium",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    training_datasets=set(),  # just contrastive pre-training
    citation="""@article{enevoldsenScandinavianEmbeddingBenchmarks2024,
    title = {The {Scandinavian} {Embedding} {Benchmarks}: {Comprehensive} {Assessment} of {Multilingual} and {Monolingual} {Text} {Embedding}},
    shorttitle = {The {Scandinavian} {Embedding} {Benchmarks}},
    url = {https://openreview.net/forum?id=pJl_i7HIA72},
    language = {en},
    urldate = {2024-04-12},
    author = {Enevoldsen, Kenneth and Kardos, Márton and Muennighoff, Niklas and Nielbo, Kristoffer},
    month = feb,
    year = {2024},
}
""",
    public_training_data=None,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class PUGGRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="PUGGRetrieval",
        description="Information Retrieval PUGG dataset for the Polish language.",
        reference="https://aclanthology.org/2024.findings-acl.652/",
        dataset={
            "path": "clarin-pl/PUGG_IR",
            "revision": "48eff464950391ce7a3d58f37355fceccf613725",
        },
        type="Retrieval",
        category="t2t",
        date=("2023-01-01", "2024-01-01"),
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        domains=["Web"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="multiple",
        bibtex_citation=r"""
@inproceedings{sawczyn-etal-2024-developing,
  address = {Bangkok, Thailand},
  author = {Sawczyn, Albert  and
Viarenich, Katsiaryna  and
Wojtasik, Konrad  and
Domoga{\l}a, Aleksandra  and
Oleksy, Marcin  and
Piasecki, Maciej  and
Kajdanowicz, Tomasz},
  booktitle = {Findings of the Association for Computational Linguistics: ACL 2024},
  doi = {10.18653/v1/2024.findings-acl.652},
  editor = {Ku, Lun-Wei  and
Martins, Andre  and
Srikumar, Vivek},
  month = aug,
  pages = {10978--10996},
  publisher = {Association for Computational Linguistics},
  title = {Developing {PUGG} for {P}olish: A Modern Approach to {KBQA}, {MRC}, and {IR} Dataset Construction},
  url = {https://aclanthology.org/2024.findings-acl.652/},
  year = {2024},
}
""",
    )
